/*
 * PowerTracer
 * http://eeclust.de
 *
 * Copyright (c) 2010-2012, University of Hamburg
 *
 * See the file COPYING in the package base directory for details
 *
 * LMG.c
 *
 *  Created on: May 11, 2011
 *  Author: Timo Minartz
 */

#include <stdlib.h>
#include <stdio.h>   /* Standard input/output definitions */
#include <unistd.h>  /* UNIX standard function definitions */
#include <assert.h>
#include <string.h>
#include <termios.h> /* POSIX terminal control definitions */
#include <fcntl.h>   /* File control definitions */

#include <glib.h>

#include "LMG.h"
#include "print.h"

void send_command(GIOChannel * channel, const gchar * command);

glong get_local_system_timestamp(void);

gchar * read_text_message(GIOChannel * channel);

void set_ranges(GIOChannel* channel, gint autorange_current,
		gfloat range_current, gint autorange_voltage, gfloat range_voltage);

void send_command(GIOChannel * channel, const gchar * command) {
	gsize bytes_written = -1;
	GError * error = NULL;
	if (g_io_channel_write_chars(channel, command, strlen(command),
			&bytes_written, &error) != G_IO_STATUS_NORMAL) {
		errormsg("Error writing %s to channel: %s\n", command, error->message);
	}
	g_io_channel_flush(channel, NULL);

	debugmsg("Written %d bytes for command \"%s\"\n", bytes_written, command);
}

glong get_local_system_timestamp(void) {
	GTimeVal gTimeVal;
	glong microseconds;

	g_get_current_time(&gTimeVal);

	microseconds = gTimeVal.tv_sec * 1000000 + gTimeVal.tv_usec;
	return microseconds;
}

gchar* read_text_message(GIOChannel * channel) {

	/**
	 * message format:
	 * "ascimessage"\n
	 */

	GString * messageString = g_string_new("");

	gchar buffer = 'X';
	// read 1 byte until we reach the terminating newline
	while (buffer != '\n') {
		g_io_channel_read_chars(channel, &buffer, 1, NULL, NULL);
		g_string_append_c(messageString, buffer);
	}

	gchar * message = g_strdup(messageString->str);

	g_string_free(messageString, TRUE);

	return message;
}

void set_ranges(GIOChannel* channel, gint autorange_current,
		gfloat range_current, gint autorange_voltage, gfloat range_voltage) {

	gint message_size = 15;
	gchar *message = (gchar*) malloc(message_size * sizeof(gchar));

	for (gint i = 1; i <= 4; ++i) {
		/* set autoranging for current */
		g_snprintf(message, message_size, "IAM%d %d\n", i, autorange_current);
		send_command(channel, message);

		/* set current range for channel */
		g_snprintf(message, message_size, "IRNG%d %1.3f\n", i, range_current);
		send_command(channel, message);

		/* set autoranging for voltage */
		g_snprintf(message, message_size, "UAM%d %d\n", i, autorange_voltage);
		send_command(channel, message);

		/* set voltage range for channel */
		g_snprintf(message, message_size, "URNG%d %3.1f\n", i, range_voltage);
		send_command(channel, message);
	}

	g_free(message);
}

void LMG_print_ranges(const LMG* lmg) {

	gint message_size = 10;
	gchar *send_message = (gchar*) malloc(message_size * sizeof(gchar));
	gchar *recv_message;

	for (gint i = 1; i <= lmg->countChannels; ++i) {

		g_print("Ranges for channel %d (%s):\n", i, lmg->names[i - 1]);

		/* get autoranging for current */

		g_snprintf(send_message, message_size, "IAM%d?\n", i);
		send_command(lmg->channel, send_message);

		recv_message = read_text_message(lmg->channel);

		gint current_autorange = atoi(recv_message);
		if (current_autorange == 0)
			g_print("Autoranging current: disabled\n");
		else
			g_print("Autoranging current: enabled\n");

		g_free(recv_message);

		/* get current range for channel */

		g_snprintf(send_message, message_size, "IRNG%d?\n", i);
		send_command(lmg->channel, send_message);

		recv_message = read_text_message(lmg->channel);

		gfloat current_range = atof(recv_message);
		g_print("Current range: %1.3f\n", current_range);

		g_free(recv_message);

		/* get autoranging for voltage */

		g_snprintf(send_message, message_size, "UAM%d?\n", i);
		send_command(lmg->channel, send_message);

		recv_message = read_text_message(lmg->channel);

		gint voltage_autorange = atoi(recv_message);
		if (voltage_autorange == 0)
			g_print("Autoranging voltage: disabled\n");
		else
			g_print("Autoranging voltage: enabled\n");

		g_free(recv_message);

		/* get voltage range for channel */

		g_snprintf(send_message, message_size, "URNG%d?\n", i);
		send_command(lmg->channel, send_message);

		recv_message = read_text_message(lmg->channel);

		gfloat voltage_range = atof(recv_message);
		g_print("Voltage range: %1.3f\n", voltage_range);

		g_free(recv_message);

	}

	g_free(send_message);
}

LMG* LMG_alloc(const gchar * port, const gint countChannels,
		const gchar ** names) {

	LMG * lmg; /* pointer to the result device */
	GIOChannel *channel; /* channel for the result device */
	int fd; /* file descriptor for the port */
	struct termios options;
	long flags;

	lmg = g_malloc(sizeof(LMG));

	lmg->countChannels = countChannels;
	lmg->names = names;

	fd = open(port, O_RDWR | O_NOCTTY | O_NONBLOCK);
	if (fd < 0) {
		g_error("open()");
		g_free(lmg);
		return NULL;
	}

	/* throw away all in and output pending on the device */
	tcflush(fd, TCIOFLUSH);

	fcntl(fd, F_SETFL, 0);

	/* 57600 baud, 8 data bits, no parity, 1 stop bit */
	flags = B57600;
	flags |= CS8;

	/* lock the file descriptor */
	if (lockf(fd, F_TLOCK, 0) != 0) {
		errormsg("Error, serial port already in use by another process.");
		g_free(lmg);
		return NULL;
	}

	/* clears all flags on the file descriptor */
	fcntl(fd, F_SETFL, 0);

	bzero(&options, sizeof(options));

	/* set raw input, 1 second timeout */
	options.c_cflag = (CLOCAL | CREAD | flags);
	options.c_iflag = IGNPAR;
	options.c_oflag = IGNPAR;
	options.c_iflag &= ~(IXON | IXOFF | IXANY);
	options.c_oflag &= ~(IXON | IXOFF | IXANY);

	/* Minimum number of characters for noncanonical read */
	options.c_cc[VMIN] = (unsigned char) 0;
	/* Timeout in deciseconds for noncanonical read. */
	options.c_cc[VTIME] = (unsigned char) 1200;

	/* set the options */
	tcsetattr(fd, TCSANOW, &options);
	tcflush(fd, TCIOFLUSH);

	/*
	 * create and initialize the GIOChannel
	 */

	channel = g_io_channel_unix_new(fd);

	g_io_channel_set_encoding(channel, NULL, NULL);
	g_io_channel_set_buffered(channel, FALSE);

	/*
	 * Set command language to short. If already short, a error (-110,"Command header error;:SYSTem:LANGuage SHORT") is added
	 * to the error queue of the device, but can be ignored.
	 */
	send_command(channel, ":SYSTem:LANGuage SHORT\n");

	lmg->channel = channel;

	LMG_set_default_ranges(lmg);

	return lmg;
}

void LMG_unref(LMG * lmg) {

	if (lmg != NULL) {

		// set back to long system message to avoid filling the error queue of the device

		send_command(lmg->channel, "LANG SCPI\n");

		g_io_channel_shutdown(lmg->channel, TRUE, NULL);

		g_io_channel_unref(lmg->channel);

		g_free(lmg);

		lmg = NULL;
	}
}

void LMG_send_command(const LMG* lmg, const gchar * command) {
	send_command(lmg->channel, command);
}

Cycle* LMG_alloc_cycle(const LMG* lmg) {
	if (lmg != NULL) {
		Cycle * cycle = g_malloc(sizeof(Cycle));
		cycle->power = g_malloc(lmg->countChannels * sizeof(gfloat));
		cycle->countValues = lmg->countChannels;
		return cycle;
	} else {
		return NULL;
	}
}

void LMG_unref_cycle(LMG * lmg, Cycle* cycle) {
	if (cycle != NULL) {
		g_free(cycle->power);
		g_free(cycle);
		cycle = NULL;
	}
}

void LMG_read_cycle(const LMG *lmg, Cycle * cycle) {

	/**
	 * ACTN;P1?;P2?;P3?;P4? ->
	 * #500016XXXXXXXXXXXXXXXX\n
	 *        <p1><p2><p3><p4>
	 *        4 x float
	 *
	 * # -> Begin of binary message
	 * 5 -> Length of length of binary message, means next 5 bytes represent the length of the message
	 * 00016 -> Length of the binary message (16 bytes)
	 * XXXXXXXXXXXXXXXX -> 16 bytes the real message
	 * \n -> terminates message
	 */

	gsize bsize = 7 + 1 + 4 * sizeof(gfloat);
	gchar buffer[bsize];

	gsize bytes_read;
	gsize bytes_total = 0;
	while (bytes_total < bsize) {
		bytes_read = -1;
		g_io_channel_read_chars(lmg->channel, buffer + bytes_total,
				bsize - bytes_total, &bytes_read, NULL);
		bytes_total += bytes_read;
	}

	gfloat tmp;

	memcpy(&tmp, buffer + 7, 4);
	cycle->power[0] = tmp;
	memcpy(&tmp, buffer + 11, 4);
	cycle->power[1] = tmp;
	memcpy(&tmp, buffer + 15, 4);
	cycle->power[2] = tmp;
	memcpy(&tmp, buffer + 19, 4);
	cycle->power[3] = tmp;

	// timestamp in microseconds
	cycle->timestamp = get_local_system_timestamp();

	debugmsg("%ld - Read Cycle (%s,%s,%s,%s) = (%3.2f,%3.2f,%3.2f,%3.2f)\n", cycle->timestamp, lmg->names[0],lmg->names[1],lmg->names[2],lmg->names[3], cycle->power[0], cycle->power[1], cycle->power[2], cycle->power[3]);
}

gchar * LMG_get_identity(const LMG * lmg) {

	send_command(lmg->channel, "*IDN?\n");

	sleep(1);

	return read_text_message(lmg->channel);
}

void LMG_set_default_ranges(const LMG * lmg) {
	set_ranges(lmg->channel, 0, 1.2, 0, 250.0);
}

gchar * LMG_get_all_errors(const LMG* lmg) {
	/*
	 * Set output format to ASCII mode
	 */
	send_command(lmg->channel, "FRMT ASCII\n");

	send_command(lmg->channel, "ERRALL?\n");

	return read_text_message(lmg->channel);

}

void LMG_beep(const LMG* lmg) {

	send_command(lmg->channel, "BEEP\n");

}

void LMG_reset(const LMG* lmg) {

	/*
	 * Reset the event register of all register structures
	 * and clear the error queue.
	 */
	send_command(lmg->channel, "*CLS\n");

	/*
	 * Reset the measuring unit.
	 */
	send_command(lmg->channel, "*RST\n");
}
