/*
 * PowerTracer
 * http://eeclust.de
 *
 * Copyright (c) 2010-2012, University of Hamburg
 *
 * See the file COPYING in the package base directory for details
 *
 * LMG.h
 *
 *  Created on: May 11, 2011
 *  Author: Timo Minartz
 */

#ifndef LMG_H_
#define LMG_H_

#include <glib.h>

typedef struct {
	glong timestamp;
	gfloat *power;
	gint countValues;
} Cycle;

typedef struct {
	GIOChannel * channel;
	const gchar **names;
	gint countChannels;
} LMG;

/**
 * Allocates a new LMG device. The port is opened and configured, the internal GIOChannel is opened.
 * The LMG System language is set to short.
 * @param port Port of the LMG (/dev/ttyUSB0, ... )
 * @param countChannels count channels of this LMG device (e.g. 4)
 * @param names Names of the channels, eg. the node names connected to each channel
 * @return Newly allocated pointer to LMG, has be freed using LMG_unref()
 */
LMG* LMG_alloc(const gchar * port, const gint countChannels, const gchar ** names);

/**
 * Allocates a measurement cycle element for this LMG device designed to contain the data of one measurement cycle
 * @param lmg LMG device to construct Cycle for
 * @return Newly allocated pointer to Cycle, has be freed using LMG_unref_cycle()
 */
Cycle* LMG_alloc_cycle(const LMG* lmg);

/**
 * Frees the internal datastructures and the device itself. The LMG System language is set to long.
 * @param lmg LMG device to free
 */
void LMG_unref(LMG * lmg);

/**
 * Frees the internal datastructures and the Cycle itself
 * @param lmg LMG device cycle belongs to
 * @param cycle Cycle to free
 */
void LMG_unref_cycle(LMG * lmg, Cycle* cycle);

/**
 * Sends a command to the specified LMG device.
 * @param lmg LMG device to sent command to
 * @param command Command to be send to device, must end with a newline
 */
void LMG_send_command(const LMG* lmg, const gchar * command);

/**
 * Reads one measurement cycle from the specified device.
 * @param lmg LMG device to read cycle from
 * @param cycle Pointer to already allocated Cycle object which will contain the measurement cycle data after the call
 */
void LMG_read_cycle(const LMG *lmg, Cycle * cycle);

/**
 * Sends the command for getting the identity from the device to test communication.
 * @param lmg LMG device to read identity from
 * @return A newly allocated pointer with the identity or NULL. The pointer has to be freed using g_free()
 */
gchar * LMG_get_identity(const LMG * lmg);

/**
 * Sends the command for setting the default ranges of the LMG device, which are:
 * * autoranging disabled for all channels
 * * voltage range to 250.0 V
 * * current range to 1.2 A
 *
 * @param lmg LMG device to set ranges of
 */
void LMG_set_default_ranges(const LMG * lmg);

/**
 * Sends the command for getting the ranges of the LMG device, which are:
 * * autoranging
 * * voltage range
 * * current range
 * and prints them to stdout.
 *
 * @param lmg LMG device to get ranges from
 */
void LMG_print_ranges(const LMG* lmg);

/**
 * Sends the command for getting all errors from the device.
 *
 * CAUTION: This method sets the device to ASCII mode independent of the current mode!
 *
 * @param lmg LMG device to get errors from
 * @return A newly allocated pointer with the errors or NULL. The pointer has to be freed using g_free()
 */
gchar * LMG_get_all_errors(const LMG* lmg);

/**
 * Sends a beep command to the device
 * @param lmg LMG device to beep
 */
void LMG_beep(const LMG* lmg);

/**
 * Sends the command to reset the event register of all register structures and to clear the error queue.
 * Further the command to reset the measuring unit is sent.
 * @param lmg LMG device to reset
 */
void LMG_reset(const LMG* lmg);

#endif /* LMG_H_ */
