/*
 * PowerTracer
 * http://eeclust.de
 *
 * Copyright (c) 2010-2012, University of Hamburg
 *
 * See the file COPYING in the package base directory for details
 *
 * main.c
 *
 *  Created on: May 11, 2011
 *  Author: Timo Minartz
 */

#include <glib.h>
#include <stdlib.h> /* for getenv */
#include <string.h> /* for strlen */
#include <unistd.h> /* for sleep */
#include "LMGSet.h"
#include "print.h"

static gboolean finalize = FALSE;

#ifdef DEBUG
gboolean debug = TRUE;
gboolean verbose = TRUE;
#else
gboolean debug = FALSE;
gboolean verbose = FALSE;
#endif

#ifdef HAVE_DBC
#include "DBConnector.h"
#endif

void start_measurement(LMGSet * set);
void stop_measurement(LMGSet * set);
void sighandler(gint signal);

void start_measurement(LMGSet * set) {

	/*
	 * Set output format to binary
	 */
	LMGSet_send_command(set, "FRMT PACKED\n");

	/*
	 * Set cycle to 100 ms
	 */
	LMGSet_send_command(set, "CYCL 0.100000\n");

	/*
	 * Program action for LMG devices
	 *
	 * Values we are going to fetch:
	 * ACTN -> Following command will be executed in continuous mode of the LMG device
	 * P1 -> Power value for channel 1
	 * ...
	 * \n -> Command end
	 */

	LMGSet_send_command(set, "ACTN;P1?;P2?;P3?;P4?\n");

	// start continous mode of LMG devices
	LMGSet_send_command(set, "CONT ON\n");
}

void stop_measurement(LMGSet * set) {

	// stop continous mode of LMG devices
	LMGSet_send_command(set, "CONT OFF\n");

	/*
	 * Set output format to ASCII mode
	 */
	LMGSet_send_command(set, "FRMT ASCII\n");

	/*
	 * Reset to local mode
	 */
	LMGSet_send_command(set, "GTL\n");
}

/**
 * Handels signal for SIGTERM and SIGINT
 */
void sighandler(gint sig) {

	if (sig == SIGTERM)
		debugmsg("Received termination signal.\n");

	if (sig == SIGINT)
		debugmsg("Received interrupt from keyboard.\n");

	/* set boolean for terminating */
	finalize = TRUE;

}

int main(int argc, char **argv) {

	/* register termination signal */
	signal(SIGTERM, sighandler);
	/* register Interrupt from keyboard for ctrl-c */
	signal(SIGINT, sighandler);

	static const char ports[3][13] = { "/dev/ttyUSB0", "/dev/ttyUSB1",
			"/dev/ttyUSB2" };
	static const gchar* names[3][4] = { { "amd2", "amd3", "amd4", "amd5" }, {
			"switch", "nas1", "intel1", "intel2" }, { "intel3", "intel4",
			"intel5", "amd1" } };

#ifndef DEBUG
	// export PT_VERBOSITY=1 to get verbosity...
	gchar *verbstr = getenv("PT_VERBOSITY");
	if (verbstr != NULL)
		sscanf(verbstr, "%d", &verbose);

	// export PT_DEBUG=1 to get debugmessages
	verbstr = getenv("PT_DEBUG");
	if (verbstr != NULL)
		sscanf(verbstr, "%d", &debug);

#endif

	printmsg("Be verbose.\n");
	debugmsg("Print debugmessages.\n");

	// Set stores all LMG devices
	LMGSet * set = LMGSet_alloc();

	if (set == NULL) {
		errormsg("Cannot allocate set.\nExit.\n;");
		return 1;
	}

	// create all LMG devices and add them to set
	for (int i = 0; i < 3; ++i) {

		const gchar *port = ports[i];
		printmsg("Initializing LMG on Port: %s\n", port);
		LMG * lmg = LMG_alloc(port, 4, names[i]);

		if (lmg == NULL) {
			errormsg("LMG could not initialized. Try reset.\n");
			LMGSet_unref(set);
			return 1;
		}

		LMGSet_add(set, lmg);
	}

	Cycle ** cycles = LMGSet_alloc_cycles(set);

	if (cycles == NULL) {
		errormsg("Cannot allocate cycles. Exit.\n");
		LMGSet_unref(set);
		return 2;
	}

#ifdef HAVE_DBC
	// init db with a buffer size of 24 => 2 measurement cycles are grouped into one query
	int ret = db_initConnection(24);
	if(ret != OK) {
		errormsg("Connection to database failed with return code %d\n", ret);
		return ret;
	}
#endif

	printmsg("Starting measurement\n");

	start_measurement(set);

#ifdef BENCH
	glong count_cycles = 0;
	glong last_measurement = 0;
	glong timestamp_first;
	glong timestamp_last = 0;
	glong timestamp_before_read, timestamp_before_push;
	glong duration_read = 0;
	glong duration_push = 0;
	glong difference_timestamps;

	GIOChannel * channel = g_io_channel_new_file("/var/log/pt/bench.txt", "w", NULL);
#endif

	// while no signals received
	while (!finalize) {

#ifdef BENCH
		timestamp_before_read = getSystemTimestamp();
#endif
		// read cycles from devices
		LMGSet_read_cycles(set, cycles);
#ifdef BENCH
		duration_read += getSystemTimestamp() - timestamp_before_read;
#endif

		glong timestamp;

		// push measurement data into database
		for (gint i = 0; i < LMGSet_get_size(set); ++i) {
			timestamp = cycles[i]->timestamp;
#ifdef BENCH
			if(count_cycles == 0)
				timestamp_first = timestamp;
#endif
			for (gint j = 0; j < cycles[i]->countValues; ++j) {
				const gchar * node = names[i][j];
				gfloat power = cycles[i]->power[j];
#ifdef HAVE_DBC
#ifdef BENCH
				timestamp_before_push = getSystemTimestamp();
#endif
				db_pushNodeMetric("power",node,timestamp,power);
#ifdef BENCH
				duration_push += getSystemTimestamp() - timestamp_before_push;
#endif
#endif
				debugmsg("db_pushNodeMetric(\"power\",%s, %lu, %f);\n", node,
						timestamp, power);
			}
		}
#ifdef BENCH
		if(count_cycles < 10 || (count_cycles >= 1000 && count_cycles <= 1010) || count_cycles % 100 == 0) {
			long difference_timestamps = timestamp - timestamp_last;
			timestamp_last = timestamp;
			glong avg_read = count_cycles == 0 ? duration_read : duration_read / (count_cycles - last_measurement);
			glong avg_push = count_cycles == 0 ? duration_push : duration_push / (count_cycles - last_measurement);
			gchar buffer[200];
			g_snprintf(buffer, 199, "cycle %d - avg read: %lld microseconds, avg push: %lld microseconds, difference to last interval: %lld microseconds, total deviation: %lld microseconds\n", count_cycles, avg_read, avg_push, difference_timestamps, (timestamp - timestamp_first)-(100000 * count_cycles));
			g_io_channel_write_chars(channel, buffer, strlen(buffer), NULL, NULL);
			last_measurement = count_cycles;
			duration_read = 0;
			duration_push = 0;
			g_io_channel_flush(channel,NULL);
		}
		count_cycles++;
#endif

	}

	printmsg("Stop measurement\n");

	stop_measurement(set);

	LMGSet_unref_cycles(set, cycles);

	LMGSet_unref(set);

#ifdef HAVE_DBC
	db_finishConnection();
#endif
#ifdef BENCH
	g_io_channel_shutdown(channel, TRUE, NULL);
#endif

	return 0;
}
