/*
 * PowerTracer
 * http://eeclust.de
 *
 * Copyright (c) 2010-2012, University of Hamburg
 *
 * See the file COPYING in the package base directory for details
 *
 * testLMG.c
 *
 *  Created on: May 11, 2011
 *  Author: Timo Minartz
 */

#include "LMGSet.h"

#ifdef HAVE_DBC
#include "DBConnector.h"
#endif

gboolean debug = TRUE;
gboolean verbose = TRUE;

int main(int argc, char **argv) {

	gchar ports[3][13] = { "/dev/ttyUSB0", "/dev/ttyUSB1", "/dev/ttyUSB2" };
	static const gchar* names[3][4] = { { "amd2", "amd3", "amd4", "amd5" }, {
				"switch", "nas1", "intel1", "intel2" }, { "intel3", "intel4",
				"intel5", "amd1" } };

	gint debug = 1;
	gint iterations = 10; // min iterations: 2

	LMGSet * set; //Set stores all LMG devices
	const gchar *port;

#ifdef HAVE_DBC
	db_initConnection(24);
#endif

	set = LMGSet_alloc();

	// create all LMG devices and add them to set
	for (int i = 0; i < 3; ++i) {

		port = ports[i];
		g_print("Initializing LMG on Port: %s\n", port);
		LMG * lmg = LMG_alloc(port, 4, names[i]);

		if (lmg == NULL) {
			g_error("LMG could not initialized. Try reset.\n");
			return 1;
		}

		LMGSet_add(set, lmg);
	}

	Cycle ** cycles = LMGSet_alloc_cycles(set);

	/*
	 * Set output format to binary
	 */
	LMGSet_send_command(set, "FRMT PACKED\n");

	/*
	 * Set cycle to 100 ms
	 */
	LMGSet_send_command(set, "CYCL 0.100000\n");

	/*
	 * Program action for LMG devices
	 *
	 * Values we are going to fetch:
	 * ACTN -> Following command will be executed in continuous mode of the LMG device
	 * P1 -> Power value for channel 1
	 * ...
	 * \n -> Command end
	 */

	LMGSet_send_command(set, "ACTN;P1?;P2?;P3?;P4?\n");

	glong sum = 0;
	glong min = 10000000;
	glong max = 0;
	glong last[3] = { 0l };
	glong count_diversity = 0;
	glong count_max = 0;
	glong count_min = 0;
	glong count_power = 0;
	glong total_duration_db_insert = 0;
	glong before_db_insert = 0;
	glong after_db_insert = 0;
	glong duration_db_insert = 0;
	glong max_duration_db_insert = 0;
	glong min_duration_db_insert = 10000000;

	// start continous mode of LMG devices
	LMGSet_send_command(set, "CONT ON\n");

	g_print("Starting %d iterations...\n", iterations);

	for (gint k = 0; k < iterations; ++k) {

		LMGSet_read_cycles(set, cycles);

		for (gint i = 0; i < 3; ++i) {
			glong timestamp = cycles[i]->timestamp;
			for (gint j = 0; j < 4; ++j) {
				const gchar * node = names[i][j];
				gfloat power = cycles[i]->power[j];
#ifdef HAVE_DBC
				before_db_insert = getSystemTimestamp();
				db_pushNodeMetric("power",node,timestamp,power);
				after_db_insert = getSystemTimestamp();
				duration_db_insert = after_db_insert - before_db_insert;
				total_duration_db_insert += duration_db_insert;
				if(duration_db_insert > max_duration_db_insert)
				max_duration_db_insert = duration_db_insert;
				if(duration_db_insert < min_duration_db_insert)
				min_duration_db_insert = duration_db_insert;
#endif
				if (debug)
					g_print("db_pushNodeMetric(\"power\",%s, %lu, %f);\n",
							node, timestamp, power);
			}
		}

		for (gint i = 0; i < 3; ++i) {

			if (k > 0) {

				// sensible checking

				if (cycles[i]->timestamp <= last[i]) {
					count_diversity++;
					continue;
				}

				glong stepsize = cycles[i]->timestamp - last[i];

				sum += stepsize;

				if (stepsize < min) {
					min = stepsize;
				}

				if (stepsize > max) {
					max = stepsize;
				}

				if (stepsize > 100000)
					count_max++;

				if (stepsize < 100000)
					count_min++;

				for (gint j = 0; j < 4; ++j) {
					if (cycles[i]->power[j] < 5 || cycles[i]->power[j] > 450) {
						g_print("%s: %lu - %f\n", names[i][j],
								cycles[i]->timestamp, cycles[i]->power[j]);
						count_power++;
					}
				}

			}

			last[i] = cycles[i]->timestamp;
		}

	}

	// stop continous mode of LMG devices
	LMGSet_send_command(set, "CONT OFF\n");

	/*
	 * Set output format to ASCII mode
	 */
	LMGSet_send_command(set, "FRMT ASCII\n");

	/*
	 * Reset to local mode
	 */
	LMGSet_send_command(set, "GTL\n");

	LMGSet_unref_cycles(set, cycles);

	LMGSet_unref(set);

#ifdef HAVE_DBC
	db_finishConnection();
#endif

	// print statistics:

	g_print(
 			"Min stepsize (ms) : %lu, max stepsize (ms) : %lu, average stepsize (ms): %lu, count diversity: %lu, count < 100: %lu, count > 100: %lu, count power invalid: %lu, Min db_insert (usec) : %lu, maxdb_insert (usec) : %lu, average db_insert (usec): %lu\n",
			(long) min / 1000, (long) max / 1000, (long) (sum / ((iterations - 1) * 3)) / 1000,
			(long) count_diversity, (long) count_min, (long) count_max, (long) count_power,
			(long) min_duration_db_insert, (long) max_duration_db_insert,
			(long) total_duration_db_insert / (iterations * 3 * 4));

	return 0;
}
